require 'spec_helper'
require 'presto/mini_magick'

describe Presto::MiniMagick::Processor do
	let(:file) { ::File.open(File.expand_path('../../fixtures/images/test.gif', __FILE__)) }
	let(:image) { ::MiniMagick::Image.open(file.path) }

	describe "#process" do
		describe "when transformation is not valid" do
			it "raises error" do
				e = proc{ Presto::MiniMagick::Processor.new(:fm => 'xml').process(image) }.must_raise(Presto::InvalidTransformation)
				e.message.must_include "fm is not in set: [\\\"jpg\\\", \\\"png\\\"]\""
			end
		end
	end

	describe "#process_crop_fit" do
		describe "w and h assigned" do
			it "resizes image to fill dimensions, and crops overlap" do
				processor = Presto::MiniMagick::Processor.new(:w => 50, :h => 100)
				processor.send(:process_crop_fit, image)
				image[:width].must_equal 50
				image[:height].must_equal 100

				image.write File.expand_path("../../processed_images/process_crop_fit-w_and_h", __FILE__)
			end
		end
	end

	describe "#process_clip_fit" do
		describe "w and h assigned" do
			it "resizes image to fit within dimensions" do
				processor = Presto::MiniMagick::Processor.new(:w => 50, :h => 50)
				processor.send(:process_clip_fit, image)
				image[:width].must_equal 50
				image[:height].must_equal 20

				image.write File.expand_path("../../processed_images/process_clip_fit-w_and_h", __FILE__)
			end
		end

		describe "w assigned" do
			it "resizes image to match width" do
				processor = Presto::MiniMagick::Processor.new(:w => 100)
				processor.send(:process_clip_fit, image)
				image[:width].must_equal 100
				image[:height].must_equal 40

				image.write File.expand_path("../../processed_images/process_clip_fit-w", __FILE__)
			end
		end

		describe "h assigned" do
			it "resizes image to match height" do
				processor = Presto::MiniMagick::Processor.new(:h => 100)
				processor.send(:process_clip_fit, image)
				image[:width].must_equal 250
				image[:height].must_equal 100

				image.write File.expand_path("../../processed_images/process_clip_fit-h", __FILE__)
			end
		end

		describe "w and h unassigned" do
			it "does nothing" do
				processor = Presto::MiniMagick::Processor.new
				processor.send(:process_clip_fit, image)
				image[:width].must_equal 500
				image[:height].must_equal 200

				image.write File.expand_path("../../processed_images/process_clip_fit-no_dimensions", __FILE__)
			end
		end
	end

	describe "#process_scale_fit" do
		describe "w and h assigned" do
			it "scales image to match dimensions" do
				processor = Presto::MiniMagick::Processor.new(:w => 50, :h => 100)
				processor.send(:process_scale_fit, image)
				image[:width].must_equal 50
				image[:height].must_equal 100

				image.write File.expand_path("../../processed_images/process_scale_fit-w_and_h", __FILE__)	
			end
		end

		describe "w assigned" do
			it "scales image to match width" do
				processor = Presto::MiniMagick::Processor.new(:w => 50)
				processor.send(:process_scale_fit, image)
				image[:width].must_equal 50
				image[:height].must_equal 200

				image.write File.expand_path("../../processed_images/process_scale_fit-w", __FILE__)
			end
		end

		describe "h assigned" do
			it "scales image to match height" do
				processor = Presto::MiniMagick::Processor.new(:h => 50)
				processor.send(:process_scale_fit, image)
				image[:width].must_equal 500
				image[:height].must_equal 50

				image.write File.expand_path("../../processed_images/process_scale_fit-h", __FILE__)
			end
		end
	end

	describe "#process_max_fit" do
		describe "w and h assigned" do
			it "resizes large image to fit within dimensions" do
				processor = Presto::MiniMagick::Processor.new(:w => 250, :h => 500)
				processor.send(:process_max_fit, image)
				image[:width].must_equal 250
				image[:height].must_equal 100

				image.write File.expand_path("../../processed_images/process_max_fit-w_and_h_1", __FILE__)
			end

			it "does not resize if smaller than dimensions" do
				processor = Presto::MiniMagick::Processor.new(:w => 1000, :h => 1200)
				processor.send(:process_max_fit, image)
				image[:width].must_equal 500
				image[:height].must_equal 200

				image.write File.expand_path("../../processed_images/process_max_fit-w_and_h_2", __FILE__)
			end
		end

		describe "w assigned" do
			it "resizes wide image to fit within width" do
				processor = Presto::MiniMagick::Processor.new(:w => 250)
				processor.send(:process_max_fit, image)
				image[:width].must_equal 250
				image[:height].must_equal 100

				image.write File.expand_path("../../processed_images/process_max_fit-w_1", __FILE__)
			end

			it "does not adjust image if less wide than width" do
				processor = Presto::MiniMagick::Processor.new(:w => 600)
				processor.send(:process_max_fit, image)
				image[:width].must_equal 500
				image[:height].must_equal 200

				image.write File.expand_path("../../processed_images/process_max_fit-w_2", __FILE__)
			end
		end

		describe "h assigned" do
			it "reduces tall image to fit within height" do
				processor = Presto::MiniMagick::Processor.new(:h => 50)
				processor.send(:process_max_fit, image)
				image[:width].must_equal 125
				image[:height].must_equal 50

				image.write File.expand_path("../../processed_images/process_max_fit-h_1", __FILE__)
			end

			it "does not adjust image if less wide than width" do
				processor = Presto::MiniMagick::Processor.new(:h => 300)
				processor.send(:process_max_fit, image)
				image[:width].must_equal 500
				image[:height].must_equal 200

				image.write File.expand_path("../../processed_images/process_max_fit-h_2", __FILE__)
			end
		end
	end

	describe "#process_fill_fit" do
		describe "w and h assigned" do
			it "resizes image to fit within dimensions, applies background to dimension" do
				processor = Presto::MiniMagick::Processor.new(:w => 400, :h => 400, :bg => 'F00')
				processor.send(:process_fill_fit, image)
				image[:width].must_equal 400
				image[:height].must_equal 400

				image.write File.expand_path("../../processed_images/process_fill_fit-w_h", __FILE__)
			end
		end
	end

	describe "#process_format" do
		it "sets image format" do
			processor = Presto::MiniMagick::Processor.new(:fm => 'png')
			processor.send(:process_format, image)
			image[:format].must_equal 'PNG'
		end
	end

	describe "#process_quality" do
		it "sets image quality" do
			img1 = ::MiniMagick::Image.open(file.path)
			processor = Presto::MiniMagick::Processor.new(:q => 10, :fm => 'jpg', :w => 50)
			processor.process img1
			img1.write File.expand_path("../../processed_images/process_quality-10", __FILE__)

			img2 = ::MiniMagick::Image.open(file.path)
			processor = Presto::MiniMagick::Processor.new(:q => 100, :fm => 'jpg', :w => 50)
			processor.process img2
			img2.write File.expand_path("../../processed_images/process_quality-100", __FILE__)
		end
	end
end